package org.gstreamer.discoverer;

import org.gstreamer.ClockTime;
import org.gstreamer.GError;
import org.gstreamer.GObject;
import org.gstreamer.GstException;
import org.gstreamer.lowlevel.GstAPI.GErrorStruct;
import org.gstreamer.lowlevel.GstDiscovererAPI;
import org.gstreamer.lowlevel.GstNative;

import com.sun.jna.Pointer;

public class Discoverer extends GObject {
    public static final String GTYPE_NAME = "GstDiscoverer";

    private static final GstDiscovererAPI gst = GstNative.load("gstpbutils", GstDiscovererAPI.class);

    public Discoverer(Initializer init) {
        super(init);
    }

    private Discoverer(ClockTime timeout, Pointer[] err) {
        this(initializer(gst.ptr_gst_discoverer_new(timeout, err), false, true));
        if (null != err[0]) {
            throw new GstException(new GError(new GErrorStruct(err[0])));
        }
    }

    /**
     * Creates a new GstDiscoverer with the provided timeout. Parameters
     *
     * timeout
     *
     *
     * timeout per file, in nanoseconds. Allowed are values between one second
     * (GST_SECOND) and one hour (3600 * GST_SECOND)
     *
     *
     * err
     *
     *
     * a pointer to a GError. can be NULL
     *
     * Returns
     *
     * The new GstDiscoverer. If an error occurred when creating the discoverer,
     * err will be set accordingly and NULL will be returned. If err is set, the
     * caller must free it when no longer needed using g_error_free().
     *
     * [transfer full]
     */
    public Discoverer(ClockTime timeout) {
        this(timeout, new Pointer[] { null });
    }

    private static final int DEFAULT_TIMEOUT_IN_S = 10;
    public Discoverer() {
        this(ClockTime.fromSeconds(DEFAULT_TIMEOUT_IN_S));
    }

    /**
     * gst_discoverer_start ()
     *
     * void gst_discoverer_start (GstDiscoverer *discoverer);
     *
     * Allow asynchronous discovering of URIs to take place. A GMainLoop must be
     * available for GstDiscoverer to properly work in asynchronous mode.
     * Parameters
     *
     * discoverer
     *
     *
     * A GstDiscoverer
     */
    public void start() {
        gst.gst_discoverer_start(this);
    }

    /**
     * Stop the discovery of any pending URIs and clears the list of pending
     * URIS (if any).
     */

    public void stop() {
        gst.gst_discoverer_stop(this);
    }

    /**
     * Synchronously discovers the given uri .
     *
     * A copy of uri will be made internally, so the caller can safely g_free()
     * afterwards. Parameters
     *
     * discoverer
     *
     *
     * A GstDiscoverer
     *
     *
     * uri
     *
     *
     * The URI to run on.
     *
     *
     * err
     *
     *
     * If an error occurred, this field will be filled in. [out][allow-none]
     * Returns
     *
     * the result of the scanning. Can be NULL if an error occurred.
     */

    public GstDiscovererInfo discoverURI(String uri) {
        Pointer []error = new Pointer[]{null};
        //        GstDiscovererInfo result = gst.gst_discoverer_discover_uri(this, uri, error);

        GstDiscovererInfo result = gst.gst_discoverer_discover_uri(this, uri, error);
        if (null != error[0]) {
            throw new GstException(new GError(new GErrorStruct(error[0])));
        }
        return result;
    }

    /**
     *
     * Appends the given uri to the list of URIs to discoverer. The actual
     * discovery of the uri will only take place if gst_discoverer_start() has
     * been called.
     *
     * A copy of uri will be made internally, so the caller can safely g_free()
     * afterwards. Parameters
     *
     * discoverer
     *
     *
     * A GstDiscoverer
     *
     *
     * uri
     *
     *
     * the URI to add.
     *
     * Returns
     *
     * TRUE if the uri was successfully appended to the list of pending uris,
     * else FALSE
     */
    public boolean discoverURIAsync(String uri) {
        return gst.gst_discoverer_discover_uri_async(this, uri);
    }
}
