package org.gstreamer.elements;

import java.util.Arrays;
import java.util.List;

import org.gstreamer.lowlevel.GstNative;

import com.sun.jna.Library;
import com.sun.jna.Pointer;
import com.sun.jna.Structure;

/**
 * gstvideosink
 *
 * gstvideosink—Base class for video sinks Functions
 */

/**
 * Properties
 *
 * gboolean show-preroll-frame Read / Write / Construct.
 * <p>
 * Whether to show video frames during preroll. If set to FALSE, video frames
 * will only be rendered in PLAYING state.
 *
 *
 */
public abstract class VideoSink extends BaseSink {
    public static final String GTYPE_NAME = "GstVideoSink";
    public static final String GST_NAME = "gstvideosink";

    private interface VideoSinkAPI extends Library {
        void gst_video_sink_center_rect(GstVideoRectangle src, GstVideoRectangle dst, GstVideoRectangle[] result,
                boolean scaling);
    };

    private static final VideoSinkAPI gst = GstNative.load("gstreamer", VideoSinkAPI.class);

    public VideoSink(Initializer init) {
        super(init);
    }

    public class GstVideoRectangle extends Structure {
 /*       struct GstVideoRectangle {
            gint x;
            gint y;
            gint w;
            gint h;
          };*/
        public int x;
        public int y;
        public int w;
        public int h;

        public GstVideoRectangle() {
        }

        public GstVideoRectangle(Pointer ptr) {
            useMemory(ptr);
            read();
        }

        @Override
        protected List<String> getFieldOrder() {
            return Arrays.asList(new String[]{"x","y","w","h"});
        }
    }

    /**
     * Takes src rectangle and position it at the center of dst rectangle with
     * or without scaling . It handles clipping if the src rectangle is bigger
     * than the dst one and scaling is set to FALSE.
     */
    public GstVideoRectangle centerRect(GstVideoRectangle src,
            GstVideoRectangle dst,
            boolean scaling) {
        GstVideoRectangle[] result = new GstVideoRectangle[] { new GstVideoRectangle() };
        gst.gst_video_sink_center_rect(src, dst, result, scaling);
        return result[0];
    }

}
