package org.gstreamer.encoding;

import java.util.List;

import org.gstreamer.Caps;
import org.gstreamer.lowlevel.GObjectAPI;
import org.gstreamer.lowlevel.GlibAPI.GList;

/**
 * GstEncodingContainerProfile
 *
 * Encoding profiles for containers. Keeps track of a list of GstEncodingProfile
 */

public class EncodingContainerProfile extends EncodingProfile {
    public static final String GTYPE_NAME = "GstEncodingContainerProfile";

    public EncodingContainerProfile(Initializer init) {
        super(init);
    }

    /**
     * Creates a new GstEncodingContainerProfile. Parameters
     *
     * name
     *
     *
     * The name of the container profile, can be NULL. [allow-none]
     *
     * description
     *
     *
     * The description of the container profile, can be NULL. [allow-none]
     *
     * format
     *
     *
     * The format to use for this profile
     *
     *
     * preset
     *
     *
     * The preset to use for this profile. [allow-none] Returns
     *
     * The newly created GstEncodingContainerProfile.
     */

    public EncodingContainerProfile(String name, String description, Caps format, String preset) {
        this(initializer(gst.ptr_gst_encoding_container_profile_new(name, description, format, preset), false, true));
    }

    /**
     *
     * Add a GstEncodingProfile to the list of profiles handled by container .
     *
     * No copy of profile will be made, if you wish to use it elsewhere after
     * this method you should increment its reference count. Parameters
     *
     * container
     *
     *
     * the GstEncodingContainerProfile to use
     *
     *
     * profile
     *
     *
     * the GstEncodingProfile to add. [transfer full] Returns
     *
     * TRUE if the stream was properly added, else FALSE.
     */
    public boolean addProfile(EncodingProfile profile) {

        GObjectAPI.GOBJECT_API.g_object_ref(profile);
        boolean result = gst.gst_encoding_container_profile_add_profile(this, profile);

        return result;
    }

    /**
     *
     * Checks if container contains a GstEncodingProfile identical to profile .
     * Parameters
     *
     * container
     *
     *
     * a GstEncodingContainerProfile
     *
     *
     * profile
     *
     *
     * a GstEncodingProfile
     *
     * Returns
     *
     * TRUE if container contains a GstEncodingProfile identical to profile ,
     * else FALSE. gst_encoding_container_profile_get_profiles ()
     */
    public boolean containsProfile(EncodingProfile profile) {
        return gst.gst_encoding_container_profile_contains_profile(this, profile);
    }

    /**
     * Parameters
     *
     * profile
     *
     * a GstEncodingContainerProfile
     *
     * Returns
     *
     * the list of contained GstEncodingProfile.
     *
     * [element-type GstPbutils.EncodingProfile][transfer none]
     */
    public List<EncodingProfile> getProfiles() {
        GList glist = gst.gst_encoding_container_profile_get_profiles(this);
        return GList.convertToListOfType(glist, EncodingProfile.class);
    }

}
