package org.gstreamer.encoding;

import org.gstreamer.lowlevel.GlibAPI.GList;
import org.gstreamer.GObject;
import org.gstreamer.lowlevel.GstEncodingProfileAPI;
import org.gstreamer.lowlevel.GstNative;
import org.gstreamer.lowlevel.annotations.CallerOwnsReturn;

import com.sun.jna.Pointer;

/**
 * GstEncodingTarget
 *
 * Collection of GstEncodingProfile for a specific target or use-case.
 *
 * When being stored/loaded, targets come from a specific category, like
 * GST_ENCODING_CATEGORY_DEVICE.
 */

public class EncodingTarget extends GObject {
    private static interface API extends GstEncodingProfileAPI {
        @CallerOwnsReturn
        Pointer ptr_gst_encoding_target_profile_new(String name, String category, String description, GList profiles);
    }

    private static final API gst = GstNative.load(API.class);

    public EncodingTarget(Initializer init) {
        super(init);
        // TODO Auto-generated constructor stub
    }

    /*
     * gst_encoding_target_new ()
     *
     * GstEncodingTarget * gst_encoding_target_new (const gchar *name, const
     * gchar *category, const gchar *description, const GList *profiles);
     *
     * Creates a new GstEncodingTarget.
     *
     * The name and category can only consist of lowercase ASCII letters for the
     * first character, followed by either lowercase ASCII letters, digits or
     * hyphens ('-').
     *
     * The category should be one of the existing well-defined categories, like
     * GST_ENCODING_CATEGORY_DEVICE, but it can be a application or user
     * specific category if needed. Parameters
     *
     * name
     *
     *
     * The name of the target.
     *
     *
     * category
     *
     *
     * The name of the category to which this target belongs. For example:
     * GST_ENCODING_CATEGORY_DEVICE. [transfer none]
     *
     * description
     *
     *
     * A description of GstEncodingTarget in the current locale. [transfer none]
     *
     * profiles
     *
     *
     * A GList of GstEncodingProfile. [transfer none][element-type
     * GstPbutils.EncodingProfile] Returns
     *
     * The newly created GstEncodingTarget or NULL if there was an error.
     *
     * [transfer full]
     */

    // FIXME: List instead GList
    public EncodingTarget(String name, String category, String description, GList profiles) {
        this(initializer(gst.ptr_gst_encoding_target_profile_new(name, category, description, profiles), false, true));
    }

    /*
     * gst_encoding_target_get_name ()
     *
     * const gchar * gst_encoding_target_get_name (GstEncodingTarget *target);
     *
     * Parameters
     *
     * target
     *
     *
     * a GstEncodingTarget
     *
     * Returns
     *
     * The name of the target .
     *
     * [transfer none] gst_encoding_target_get_category ()
     *
     * const gchar * gst_encoding_target_get_category (GstEncodingTarget
     * *target);
     *
     * Parameters
     *
     * target
     *
     *
     * a GstEncodingTarget
     *
     * Returns
     *
     * The category of the target . For example: GST_ENCODING_CATEGORY_DEVICE.
     *
     * [transfer none] gst_encoding_target_get_description ()
     *
     * const gchar * gst_encoding_target_get_description (GstEncodingTarget
     * *target);
     *
     * Parameters
     *
     * target
     *
     *
     * a GstEncodingTarget
     *
     * Returns
     *
     * The description of the target .
     *
     * [transfer none] gst_encoding_target_get_profiles ()
     *
     * const GList * gst_encoding_target_get_profiles (GstEncodingTarget
     * *target);
     *
     * Parameters
     *
     * target
     *
     *
     * a GstEncodingTarget
     *
     * Returns
     *
     * A list of GstEncodingProfile(s) this target handles.
     *
     * [transfer none][element-type GstPbutils.EncodingProfile]
     * gst_encoding_target_get_profile ()
     *
     * GstEncodingProfile * gst_encoding_target_get_profile (GstEncodingTarget
     * *target, const gchar *name);
     *
     * Parameters
     *
     * target
     *
     *
     * a GstEncodingTarget
     *
     *
     * name
     *
     *
     * the name of the profile to retrieve
     *
     * Returns
     *
     * The matching GstEncodingProfile, or NULL.
     *
     * [transfer full] gst_encoding_target_add_profile ()
     *
     * gboolean gst_encoding_target_add_profile (GstEncodingTarget *target,
     * GstEncodingProfile *profile);
     *
     * Adds the given profile to the target . Each added profile must have a
     * unique name within the profile.
     *
     * The target will steal a reference to the profile . If you wish to use the
     * profile after calling this method, you should increase its reference
     * count. Parameters
     *
     * target
     *
     *
     * the GstEncodingTarget to add a profile to
     *
     *
     * profile
     *
     *
     * the GstEncodingProfile to add. [transfer full] Returns
     *
     * TRUE if the profile was added, else FALSE. gst_encoding_target_save ()
     *
     * gboolean gst_encoding_target_save (GstEncodingTarget *target, GError
     * **error);
     *
     * Saves the target to a default user-local directory. Parameters
     *
     * target
     *
     *
     * a GstEncodingTarget
     *
     *
     * error
     *
     *
     * If an error occured, this field will be filled in.
     *
     * Returns
     *
     * TRUE if the target was correctly saved, else FALSE.
     * gst_encoding_target_save_to_file ()
     *
     * gboolean gst_encoding_target_save_to_file (GstEncodingTarget *target,
     * const gchar *filepath, GError **error);
     *
     * Saves the target to the provided file location. Parameters
     *
     * target
     *
     *
     * a GstEncodingTarget
     *
     *
     * filepath
     *
     *
     * the location to store the target at.
     *
     *
     * error
     *
     *
     * If an error occured, this field will be filled in.
     *
     * Returns
     *
     * TRUE if the target was correctly saved, else FALSE.
     * gst_encoding_target_load ()
     *
     * GstEncodingTarget * gst_encoding_target_load (const gchar *name, const
     * gchar *category, GError **error);
     *
     * Searches for the GstEncodingTarget with the given name, loads it and
     * returns it.
     *
     * If the category name is specified only targets from that category will be
     * searched for. Parameters
     *
     * name
     *
     *
     * the name of the GstEncodingTarget to load.
     *
     *
     * category
     *
     *
     * the name of the target category, like GST_ENCODING_CATEGORY_DEVICE. Can
     * be NULL. [allow-none]
     *
     * error
     *
     *
     * If an error occured, this field will be filled in.
     *
     * Returns
     *
     * The GstEncodingTarget if available, else NULL.
     *
     * [transfer full] gst_encoding_target_load_from_file ()
     *
     * GstEncodingTarget * gst_encoding_target_load_from_file (const gchar
     * *filepath, GError **error);
     *
     * Opens the provided file and returns the contained GstEncodingTarget.
     * Parameters
     *
     * filepath
     *
     *
     * The file location to load the GstEncodingTarget from
     *
     *
     * error
     *
     *
     * If an error occured, this field will be filled in.
     *
     * Returns
     *
     * The GstEncodingTarget contained in the file, else NULL.
     *
     * [transfer full]
     */
}
