package org.gstreamer.encoding;

import org.gstreamer.Caps;

/**
 * Variant of GstEncodingProfile for video streams, allows specifying the pass.
 */
public class EncodingVideoProfile extends EncodingProfile {
    public static final String GTYPE_NAME = "GstEncodingVideoProfile";

    public EncodingVideoProfile(Initializer init) {
        super(init);
    }

    /**
     * * Creates a new GstEncodingVideoProfile
     *
     * All provided allocatable arguments will be internally copied, so can be
     * safely freed/unreferenced after calling this method.
     *
     * If you wish to control the pass number (in case of multi-pass scenarios),
     * please refer to the gst_encoding_video_profile_set_pass() documentation.
     *
     * If you wish to use/force a constant framerate please refer to the
     * gst_encoding_video_profile_set_variableframerate() documentation.
     * Parameters
     *
     * format
     *
     *
     * the GstCaps
     *
     *
     * preset
     *
     *
     * the preset(s) to use on the encoder, can be NULL. [allow-none]
     *
     * restriction
     *
     *
     * the GstCaps used to restrict the input to the encoder, can be NULL. See
     * gst_encoding_profile_get_restriction() for more details. [allow-none]
     *
     * presence
     *
     *
     * the number of time this stream must be used. 0 means any number of times
     * (including never)
     *
     * Returns
     *
     * the newly created GstEncodingVideoProfile.
     */
    public EncodingVideoProfile(Caps format, String preset, Caps restriction, int presence) {
        this(initializer(gst.ptr_gst_encoding_video_profile_new(format, preset, restriction, presence), false, true));
    }

    /**
     * Get the pass number if this is part of a multi-pass profile. Parameters
     *
     * prof
     *
     *
     * a GstEncodingVideoProfile
     *
     * Returns
     *
     * The pass number. Starts at 1 for multi-pass. 0 if this is not a
     * multi-pass profile
     */
    public int getPass() {
        return gst.gst_encoding_video_profile_get_pass(this);
    }

    /**
     * Parameters
     *
     * prof
     *
     *
     * a GstEncodingVideoProfile
     *
     * Returns
     *
     * Whether non-constant video framerate is allowed for encoding.
     */
    public boolean getVariableFramerate() {
        return gst.gst_encoding_video_profile_get_variableframerate(this);
    }

    /**
     * Sets the pass number of this video profile. The first pass profile should
     * have this value set to 1. If this video profile isn't part of a
     * multi-pass profile, you may set it to 0 (the default value). Parameters
     *
     * prof
     *
     *
     * a GstEncodingVideoProfile
     *
     *
     * pass
     *
     *
     * the pass number for this profile
     */
    public void setPass(int pass) {
        gst.gst_encoding_video_profile_set_pass(this, pass);
    }

    /**
     * If set to TRUE, then the incoming stream will be allowed to have
     * non-constant framerate. If set to FALSE (default value), then the
     * incoming stream will be normalized by dropping/duplicating frames in
     * order to produce a constance framerate. Parameters
     *
     * prof
     *
     *
     * a GstEncodingVideoProfile
     *
     *
     * variableframerate
     *
     *
     * a boolean
     */
    public void setVariableFramerate(boolean variableframerate) {
        gst.gst_encoding_video_profile_set_variableframerate(this, variableframerate);

    }

}
