package org.gstreamer.ges;

import java.util.concurrent.atomic.AtomicInteger;
import java.util.logging.Level;
import java.util.logging.Logger;

import org.gstreamer.Gst;
import org.gstreamer.Gst.NativeArgs;
import org.gstreamer.ges.clip.GESBaseTransitionClip;
import org.gstreamer.ges.clip.GESClip;
import org.gstreamer.ges.clip.GESOperationClip;
import org.gstreamer.ges.clip.GESOverlayClip;
import org.gstreamer.ges.clip.GESSourceClip;
import org.gstreamer.ges.clip.GESTestClip;
import org.gstreamer.ges.clip.GESTextOverlayClip;
import org.gstreamer.ges.clip.GESTitleClip;
import org.gstreamer.ges.clip.GESTransitionClip;
import org.gstreamer.ges.clip.GESUriClip;
import org.gstreamer.ges.operation.GESAudioTransition;
import org.gstreamer.ges.operation.GESBaseEffect;
import org.gstreamer.ges.operation.GESEffect;
import org.gstreamer.ges.operation.GESOperation;
import org.gstreamer.ges.operation.GESTextOverlay;
import org.gstreamer.ges.operation.GESTransition;
import org.gstreamer.ges.operation.GESVideoTransition;
import org.gstreamer.ges.source.GESAudioSource;
import org.gstreamer.ges.source.GESAudioTestSource;
import org.gstreamer.ges.source.GESAudioUriSource;
import org.gstreamer.ges.source.GESImageSource;
import org.gstreamer.ges.source.GESMultiFileSource;
import org.gstreamer.ges.source.GESSource;
import org.gstreamer.ges.source.GESTitleSource;
import org.gstreamer.ges.source.GESVideoSource;
import org.gstreamer.ges.source.GESVideoUriSource;
import org.gstreamer.Version;
import org.gstreamer.lowlevel.GESAPI;
import org.gstreamer.lowlevel.GlibAPI;
import org.gstreamer.lowlevel.GstNative;
import org.gstreamer.lowlevel.NativeObject;

import com.sun.jna.Pointer;

public class GES {
    private static Logger logger = Logger.getLogger(Gst.class.getName());

    private static final AtomicInteger initCount = new AtomicInteger(0);
    private static final API ges;

    private static interface API extends GlibAPI, GESAPI {
    }

    static {
        API lib = null;
        try {
            lib = GstNative.load("ges", API.class);
        } catch (UnsatisfiedLinkError ule) {
            ule.printStackTrace();
        }
        ges = lib;
    }
    /* no instance of GES */
    private GES() {
    }

    public static void init() {
        init(new String[] {});
    }

    public static void init(String args[]) {
        init("unknown", new String[] {});
    }

    public static String[] init(String progname, String args[]) {
        if (initCount.getAndIncrement() > 0) {
            return args;
        }
        NativeArgs argv = new NativeArgs(progname, args);

        Pointer[] error = { null };
        // FIXME: ges_init_check not available in GES 1.4
        // if (!ges.ges_init_check(argv.argcRef, argv.argvRef, error)) {
        // initCount.decrementAndGet();
        // throw new GstException(new GError(new GErrorStruct(error[0])));
        // }
        if (!ges.ges_init()) {
            throw new IllegalStateException("ges init failed");
        }
        if (logger.isLoggable(Level.FINE)) {
            logger.fine("Ges after gst_init, argc=" + argv.argcRef.getValue());
        }
        loadAllClasses();
        return argv.toStringArray();
    }

    public static Version getVersion() {
        long[] major = { 0 }, minor = { 0 }, micro = { 0 }, nano = { 0 };
        if (initCount.get() > 0 ) {
            ges.ges_version(major, minor, micro, nano);
        }
        return new Version(major[0], minor[0], micro[0], nano[0]);
    }

    public static synchronized final boolean isInitialized() {
        return initCount.get() > 0;
    }

    @SuppressWarnings("unchecked")
    private static synchronized void loadAllClasses() {
        for (Class<?> cls : nativeClasses) {
            Gst.registerClass((Class<? extends NativeObject>) cls);
        }
    }

    @SuppressWarnings("rawtypes")
    private static Class[] nativeClasses = {
            GESTrackElement.class,
            GESTimelineElement.class,
            GESAudioTestSource.class,
            GESAudioUriSource.class,
            GESAudioSource.class,
            GESSource.class,
            GESVideoSource.class,
            GESVideoUriSource.class,
            GESImageSource.class,
            GESMultiFileSource.class,
            GESTitleSource.class,
            GESVideoTestSource.class,
            GESOperation.class,
            GESAudioTransition.class,
            GESBaseEffect.class,
            GESEffect.class,
            GESTextOverlay.class,
            GESTransition.class,
            GESVideoTransition.class,
            GESContainer.class,
    //      clip
            GESOperationClip.class,
            GESBaseTransitionClip.class,
            GESClip.class,
            GESOperationClip.class,
            GESOverlayClip.class,
            GESSourceClip.class,
            GESTestClip.class,
            GESTextOverlayClip.class,
            GESTitleClip.class,
            GESTransitionClip.class,
            GESUriClip.class
    };
}
