package org.gstreamer.ges;

import java.util.List;

import org.gstreamer.lowlevel.GESContainerAPI;
import org.gstreamer.lowlevel.GlibAPI.GList;

/**
 *
 * GESContainer
 *
 * GESContainer — Base Class for objects responsible for controlling other
 * GESTimelineElement-s
 */

public class GESContainer extends GESTimelineElement {
    public static final String GTYPE_NAME = "GESContainer";

    private static GESContainerAPI ges = GESContainerAPI.GSTCONTAINER_API;

    public GESContainer(Initializer init) {
        super(init);
    }

    /**
     * Get the list of GESTimelineElement contained in container The user is
     * responsible for unreffing the contained objects and freeing the list.
     * Parameters
     *
     * container
     *
     *
     * a GESContainer
     *
     *
     * recursive
     *
     *
     * Whether to recursively get children in container
     *
     * Returns
     *
     * The list of timeline element contained in container .
     *
     * [transfer full][element-type GESTimelineElement]
     */
    public List<GESTimelineElement> getChildren(boolean recursive) {
        GList glist = ges.ges_container_get_children(this, recursive);
        return GList.convertToListOfType(glist, GESTimelineElement.class, true);
    }

    /**
    Add the GESTimelineElement39 to the container.
    Parameters

    container


    a GESContainer40


    child


    the GESTimelineElement41

    Returns

    TRUE42 on success, FALSE43 on failure.
    */
    public boolean add(GESTimelineElement child) {
        return ges.ges_container_add(this, child);
    }

    /**
     *     Release the child from the control of container .
    Parameters

    container


    a GESContainer


    child


    the GESTimelineElement to release

    Returns

    TRUE if the child was properly released, else FALSE.
    */

    public void remove(GESTimelineElement child) {
        ges.ges_container_remove(this, child);
    }

    /**
     * Ungroups the GESTimelineElement contained in this GESContainer, creating
     * new GESContainer55 containing those GESTimelineElement apropriately.
     * Parameters
     *
     * container
     *
     *
     * The GESContainer to ungroup. [transfer full]
     *
     * recursive
     *
     *
     * Wether to recursively ungroup container
     *
     * Returns
     *
     * The list of GESContainer resulting from the ungrouping operation The user
     * is responsible for unreffing the contained objects and freeing the list.
     *
     * [transfer full][element-type GESContainer]
     */
    public List<GESContainer> ungroup(boolean recursive) {
        GList glist = ges.ges_container_ungroup(this, recursive);
        return GList.convertToListOfType(glist, GESContainer.class, true);
    }

    /**
     *
     * Groups the GESContainer-s provided in containers . It creates a subclass
     * of GESContainer, depending on the containers provided in containers .
     * Basically, if all the containers in containers should be contained in a
     *
     * same clip (all the GESTrackElement they contain have the exact same
     * start/inpoint/duration and are in the same layer), it will create a
     * GESClip otherwise a GESGroup will be created Parameters
     *
     * containers
     *
     *
     * The GESContainer to group, they must all be in a same GESTimeline.
     * [transfer none][element-type GESContainer][allow-none] Returns
     *
     * The GESContainer (subclass) resulting of the grouping.
     *
     * [transfer none]
     */
    public static GESContainer group(List<GESContainer> containers) {
        GList glist = new GList();
        //TODO: implement
        for (GESContainer container:containers) {
            //            glist.a
        }
        return null;
    }

    /*
    *
    *  Types and Values GESContainer
    *
    * typedef struct { /*< readonly >
    */
    //GList*children;

    /* We don't add those properties to the priv struct for optimization and code
     * readability purposes * /
    guint32 height; /* the span of priorities this object needs * /
    }GESContainer;

    The GESContainer68 base class.
    Members

    GList69*children;

    A list
    of TimelineElement
    controlled by this Container.NOTE:
    Do not modify.[element-
    type GES.TimelineElement]

    guint3270 height;

    The span
    of priorities this
    container occupies

    GESContainerClass

    typedef struct

    {
      /* signals * /
      void (*child_added)             (GESContainer *container, GESTimelineElement *element);
      void (*child_removed)           (GESContainer *container, GESTimelineElement *element);
      gboolean (*add_child)           (GESContainer *container, GESTimelineElement *element);
      gboolean (*remove_child)        (GESContainer *container, GESTimelineElement *element);
      GList* (*ungroup)               (GESContainer *container, gboolean recursive);
      GESContainer * (*group)         (GList *containers);
      gboolean (*edit)                (GESContainer * container,
                                       GList * layers, gint new_layer_priority,
                                       GESEditMode mode,
                                       GESEdge edge,
                                       guint64 position);
    }GESContainerClass;

    Members

    child_added ()


    Virtual method that is called right after a GESTimelineElement71 is added


    child_removed ()


    Virtual method that is called right after a GESTimelineElement72 is removed


    add_child ()


    Virtual method to add a child


    remove_child ()


    Virtual method to remove a child


    ungroup ()


    Ungroups the GESTimelineElement73 contained in this GESContainer74, creating new


    group ()


    Groups the GESContainers75 together GESContainer76 containing those GESTimelineElement77 apropriately.


    edit ()
    */

}
