package org.gstreamer.ges;

import java.util.List;

import org.gstreamer.Bin;
import org.gstreamer.Caps;
import org.gstreamer.lowlevel.GESTrackAPI;
import org.gstreamer.lowlevel.GlibAPI;
import org.gstreamer.lowlevel.GlibAPI.GList;
import org.gstreamer.lowlevel.GstBinAPI;
import org.gstreamer.lowlevel.GstElementAPI;
import org.gstreamer.lowlevel.GstNative;
import org.gstreamer.lowlevel.annotations.CallerOwnsReturn;

import com.sun.jna.Pointer;

/**
 * GESTrack
 *
 * GESTrack—Composition of objects
 *
 * Description
 *
 * Corresponds to one output format(i.e.audio OR video).
 *
 * Contains the compatible TrackElement(s).
 *
 */
/**
 *
 * Members
 *
 * GESTrackType type;
 *
 * a GESTrackType indicting the basic type of the track.
 *
 * Property Details The“caps”property
 *
 * “caps”GstCaps*
 *
 * Caps used to filter/choose the output stream.This is generally set to a
 * generic set of caps like'video/x-raw'for raw video.
 *
 * Default value:GST_CAPS_ANY.
 *
 * Flags:Read/Write/Construct Only The“duration”property
 *
 * “duration”guint64
 *
 * Current duration of the track
 *
 * Default value:O
 *
 * Flags:Read
 *
 * Default value:1000000000 The“mixing”property
 *
 * “mixing”gboolean
 *
 * Whether layer mixing is activated or not on the track.
 *
 * Flags:Read/Write/Construct
 *
 * Default value:TRUE The“restriction-caps”property
 *
 * “restriction-caps”GstCaps*
 *
 * Caps used to filter/choose the output stream.
 *
 * Default value:GST_CAPS_ANY.
 *
 * Flags:Read/Write The“track-type”property
 *
 * “track-type”GESTrackType
 *
 * Type of stream the track outputs.This is used when creating the GESTrack to
 * specify in generic terms what type of content will be outputted.
 *
 * It also serves as a'fast'way to check what type of data will be outputted
 * from the GESTrack without having to actually check the GESTrack'scaps
 * property.
 *
 * Flags:Read/Write/Construct Only
 *
 * Default value:GES_TRACK_TYPE_CUSTOM Signal Details The“commited”signal
 *
 * void user_function(GESTrack*track,gpointer user_data)
 *
 * Parameters
 *
 * track
 *
 * the GESTrack
 *
 * user_data
 *
 * user data set when the signal handler was connected.
 *
 * Flags:Run Last The“track-element-added”signal
 *
 * void user_function(GESTrack*object,GESTrackElement*effect,gpointer user_data)
 *
 * Will be emitted after a track element was added to the track.Parameters
 *
 * object
 *
 * the GESTrack
 *
 * effect
 *
 * the GESTrackElement that was added.
 *
 * user_data
 *
 * user data set when the signal handler was connected.
 *
 * Flags:Run First The“track-element-removed”signal
 *
 * void user_function(GESTrack*object,GESTrackElement*effect,gpointer user_data)
 *
 * Will be emitted after a track element was removed from the track.Parameters
 *
 * object
 *
 * the GESTrack
 *
 * effect
 *
 * the GESTrackElement that was removed.
 *
 * user_data
 *
 * user data set when the signal handler was connected.
 *
 * Flags:Run First
 */



public class GESTrack extends Bin {
    private static interface API extends GstElementAPI, GstBinAPI, GESTrackAPI {
        @CallerOwnsReturn
        Pointer ptr_ges_track_new(GESTrackType type, Caps caps);
    }

    private static final API ges = GstNative.load("ges", API.class);

    public GESTrack(Initializer initializer) {
        super(initializer);
    }

    /**
     * Creates a new GESTrack with the given type and caps.
     *
     * The newly created track will steal a reference to the caps.If you wish to
     * use those caps elsewhere,you will have to take an extra
     * reference.Parameters
     *
     * type
     *
     * The type of track
     *
     * caps
     *
     * The caps to restrict the output of the track to.[transfer full]
     * 
     * Returns
     */

    public GESTrack(GESTrackType type, Caps caps) {
        this(initializer(ges.ptr_ges_track_new(type, caps), false));
        if (null!=caps) {
            caps.ref();
        }
    }

    /**
     *
     * Adds the given object to the track.Sets the object'scontrolling track,and
     * thus takes ownership of the object.
     *
     * An object can only be added to one track.Parameters
     *
     * track
     *
     * a GESTrack
     *
     * object
     *
     * the GESTrackElement to add.[transfer full]Returns
     *
     * TRUE if the object was properly added.FALSE if the track does not want to
     * accept the object.ges_track_set_restriction_caps()
     */
    public boolean addElement(GESTrackElement object) {
        return ges.ges_track_add_element(this, object);
    }

    /**
    *
    * Sets the given caps as the caps the track has to output.Parameters
    *
    * track
    *
    * a GESTrack
    *
    * caps
    *
    * the GstCaps to set
    */
    public void setRestrictionCaps(Caps caps) {
        ges.ges_track_set_restriction_caps(this, caps);
    }

    /**
     *
     *
     * Updates the restriction caps by modifying all the fields present in caps
     * in the original restriction caps.If for example the current restriction
     * caps are video/x-raw,format=I420,width=360 and caps is
     * video/x-raw,format=RGB,the restriction caps will be updated to
     * video/x-raw,format=RGB,width=360.
     *
     * Modification happens for each structure in the new caps,and one can add
     * new fields or structures through that function.Parameters
     *
     * track
     *
     * a GESTrack
     *
     * caps
     *
     * the GstCaps to update with
     */
    public void updateRestrictionCaps(Caps caps) {
        ges.ges_track_update_restriction_caps(this, caps);
    }

    /**
    *
    *
    *
    * Removes the object from the track and unparents it.Unparenting it means the
    * reference owned by track on the object will be removed.If you wish to use the
    * object after this function,make sure you call gst_object_ref()before removing
    * it from the track.Parameters
    *
    * track
    *
    * a GESTrack
    *
    * object
    *
    * the GESTrackElement to remove
    *
    * Returns
    *
    * TRUE if the object was removed,else FALSE if the track could not remove the
    * object(like if it didn'tbelong to the track).ges_track_get_caps()
    */
    public boolean removeElement(GESTrackElement object) {
        return ges.ges_track_remove_element(this, object);
    }

    /**
     *
     * Get the GstCaps this track is configured to output.Parameters
     *
     * track
     *
     * a GESTrack
     *
     * Returns
     *
     * The GstCaps this track is configured to output.ges_track_get_elements()
     */
    public Caps getCaps() {
        return ges.ges_track_get_caps(this);
    }

    /**
    *
    * Gets the GESTrackElement contained in track Parameters
    *
    * track
    *
    * a GESTrack
    *
    * Returns
    *
    * the list of GESTrackElement present in the Track sorted by priority and
    * start.
    *
    * [transfer full][element-type GESTrackElement]Types and Values GESTrack
    */
    public List<GESTrackElement> getTrackElements() {
        GList glist = ges.ges_track_get_elements(this);
        List<GESTrackElement> list = GList.convertToListOfType(glist, GESTrackElement.class);
        GlibAPI.GLIB_API.g_list_free(glist);
        return list;
    }

}
