package org.gstreamer.ges;

import org.gstreamer.lowlevel.IntegerEnum;

/**
 * GESTrackType:
 * @GES_TRACK_TYPE_UNKNOWN: A track of unknown type (i.e. invalid)
 * @GES_TRACK_TYPE_AUDIO: An audio track
 * @GES_TRACK_TYPE_VIDEO: A video track
 * @GES_TRACK_TYPE_TEXT: A text (subtitle) track
 * @GES_TRACK_TYPE_CUSTOM: A custom-content track
 *
 * Types of content handled by a track. If the content is not one of
 * @GES_TRACK_TYPE_AUDIO, @GES_TRACK_TYPE_VIDEO or @GES_TRACK_TYPE_TEXT,
 * the user of the #GESTrack must set the type to @GES_TRACK_TYPE_CUSTOM.
 *
 * @GES_TRACK_TYPE_UNKNOWN is for internal purposes and should not be used
 * by users
 */

public enum GESTrackType implements IntegerEnum {
    /**
     * GES_TRACK_TYPE_UNKNOWN is for internal purposes and should not be used by
     * users
     */
    UNKNOWN(1 << 0),
    /** An audio track */
    AUDIO(1 << 1),
    /** An video track */
    VIDEO(1 << 2),
    /** A text (subtitle) track */
    TEXT(1<<3),

    /** A custom-content track*/
    CUSTOM(1<<4);

    private int value;

    private GESTrackType(int value) {
        this.value = value;
    }

    @Override
    public int intValue() {
        return value;
    }
};
