package org.gstreamer.ges;
import java.util.EnumSet;

import org.gstreamer.lowlevel.IntegerEnum;

public enum GESVideoStandardTransitionType implements IntegerEnum {

    /** Transition type has not been set */
    NONE(0),

    /** A bar moves from left to right */
    BAR_WIPE_LR,
    /** A bar moves from top to bottom */
    BAR_WIPE_TB,
    /** A box expands from the upper-left corner to the lower-right corner */
    BOX_WIPE_TL,
    /** A box expands from the upper-right corner to the lower-left corner */
    BOX_WIPE_TR,
    /** A box expands from the lower-right corner to the upper-left corner */
    BOX_WIPE_BR,
    /** A box expands from the lower-left corner to the upper-right corner */
    BOX_WIPE_BL,
    /** A box shape expands from each of the four corners toward the center */
    FOUR_BOX_WIPE_CI,
    /**
     * A box shape expands from the center of each quadrant toward the corners
     * of each quadrant
     */
    FOUR_BOX_WIPE_CO,

    /**
     * A central, vertical line splits and expands toward the left and right
     * edges
     */
    BARNDOOR_V(21),
    /**
     * A central, horizontal line splits and expands toward the top and bottom
     * edges
     */
    BARNDOOR_H,

    /** A box expands from the top edge's midpoint to the bottom corners */
    BOX_WIPE_TC,

    /** A box expands from the right edge's midpoint to the left corners */
    BOX_WIPE_RC,

    /** A box expands from the bottom edge's midpoint to the top corners */
    BOX_WIPE_BC,

    /** A box expands from the left edge's midpoint to the right corners */
    BOX_WIPE_LC,

    /**
     * A diagonal line moves from the upper-left corner to the lower-right
     * corner
     */
    DIAGONAL_TL(41),


    /**
     * A diagonal line moves from the upper right corner to the lower-left
     * corner
     */
    DIAGONAL_TR,


    /**
     * Two wedge shapes slide in from the top and bottom edges toward the center
     */

    BOWTIE_V,


    /**
     * Two wedge shapes slide in from the left and right edges toward the center
     */

    BOWTIE_H,

    /**
     * A diagonal line from the lower-left to upper-right corners splits and
     * expands toward the opposite corners
     */
    BARNDOOR_DBL,

    /**
     * A diagonal line from upper-left to lower-right corners splits and expands
     * toward the opposite corners
     */
    BARNDOOR_DTL,

    /**
     * Four wedge shapes split from the center and retract toward the four edges
     */
    MISC_DIAGONAL_DBD,

    /**
     * A diamond connecting the four edge midpoints simultaneously contracts
     * toward the center and expands toward the edges
     */
    MISC_DIAGONAL_DD,

    /** A wedge shape moves from top to bottom */
    VEE_D(61),

    /** A wedge shape moves from right to left */
    VEE_L,

    /** A wedge shape moves from bottom to top */
    VEE_U,

    /** A wedge shape moves from left to right */
    VEE_R,

    /**
     * A 'V' shape extending from the bottom edge's midpoint to the opposite
     * corners contracts toward the center and expands toward the edges
     */
    BARNVEE_D,

    /**
     * A 'V' shape extending from the left edge's midpoint to the opposite
     * corners contracts toward the center and expands toward the edges
     */
    BARNVEE_L,

    /**
     * A 'V' shape extending from the top edge's midpoint to the opposite
     * corners contracts toward the center and expands toward the edges
     */
    BARNVEE_U,

    /**
     * A 'V' shape extending from the right edge's midpoint to the opposite
     * corners contracts toward the center and expands toward the edges
     */
    BARNVEE_R,

    /** A rectangle expands from the center. */
    IRIS_RECT(101),

    /** A radial hand sweeps clockwise from the twelve o'clock position */
    CLOCK_CW12(201),

    /** A radial hand sweeps clockwise from the three o'clock position */
    CLOCK_CW3,

    /** A radial hand sweeps clockwise from the six o'clock position */
    CLOCK_CW6,

    /** A radial hand sweeps clockwise from the nine o'clock position */
    CLOCK_CW9,

    /**
     * Two radial hands sweep clockwise from the twelve and six o'clock
     * positions
     */
    PINWHEEL_TBV,


    /**
     * Two radial hands sweep clockwise from the nine and three o'clock
     * positions
     */
    PINWHEEL_TBH,

    /** Four radial hands sweep clockwise */
    PINWHEEL_FB,

    /** A fan unfolds from the top edge, the fan axis at the center */
    FAN_CT(211),

    /** A fan unfolds from the right edge, the fan axis at the center */
    FAN_CR,

    /** Two fans, their axes at the center, unfold from the top and bottom */
    DOUBLEFAN_FOV,

    /** Two fans, their axes at the center, unfold from the left and right */
    DOUBLEFAN_FOH,

    /** A radial hand sweeps clockwise from the top edge's midpoint */
    SINGLESWEEP_CWT(221),

    /** A radial hand sweeps clockwise from the right edge's midpoint */
    SINGLESWEEP_CWR,

    /** A radial hand sweeps clockwise from the bottom edge's midpoint */
    SINGLESWEEP_CWB,

    /** A radial hand sweeps clockwise from the left edge's midpoint */
    SINGLESWEEP_CWL,
/*
    */

    /**
     * Two radial hands sweep clockwise and counter-clockwise from the top and
     * bottom edges' midpoints
     */
    DOUBLESWEEP_PV,

    /**
     * Two radial hands sweep clockwise and counter-clockwise from the left and
     * right edges' midpoints
     */
    DOUBLESWEEP_PD,

    /**
     * Two radial hands attached at the top and bottom edges' midpoints sweep
     * from right to left
     */
    DOUBLESWEEP_OV,

    /**
     * Two radial hands attached at the left and right edges' midpoints sweep
     * from top to bottom
     */
    DOUBLESWEEP_OH,

    /**
     * A fan unfolds from the bottom, the fan axis at the top edge's midpoint
     */
    FAN_T(231),

    /**
     * A fan unfolds from the left, the fan axis at the right edge's midpoint
     */
    FAN_R,

    /**
     * A fan unfolds from the top, the fan axis at the bottom edge's midpoint
     */
    FAN_B,

    /**
     * A fan unfolds from the right, the fan axis at the left edge's midpoint
     */
    FAN_L,

    /** Two fans, their axes at the top and bottom, unfold from the center */
    DOUBLEFAN_FIV,

    /** Two fans, their axes at the left and right, unfold from the center */
    DOUBLEFAN_FIH,


    /** A radial hand sweeps clockwise from the upper-left corner */

    SINGLESWEEP_CWTL(241),



    /** A radial hand sweeps counter-clockwise from the lower-left corner */
    SINGLESWEEP_CWBL,

    /** A radial hand sweeps clockwise from the lower-right corner */
    SINGLESWEEP_CWBR,



    /** A radial hand sweeps counter-clockwise from the upper-right corner */
    SINGLESWEEP_CWTR,

    /**
     * Two radial hands attached at the upper-left and lower-right corners sweep
     * down and up
     */
    DOUBLESWEEP_PDTL,

    /**
     * Two radial hands attached at the lower-left and upper-right corners sweep
     * down and up
     */
    DOUBLESWEEP_PDBL,

    /**
     * Two radial hands attached at the upper-left and upper-right corners sweep
     * down
     */
    SALOONDOOR_T(251),

    /**
     * Two radial hands attached at the upper-left and lower-left corners sweep
     * to the right
     */
    SALOONDOOR_L,

    /**
     * Two radial hands attached at the lower-left and lower-right corners sweep
     * up
     */
    SALOONDOOR_B,

    /**
     * Two radial hands attached at the upper-right and lower-right corners
     * sweep to the left
     */
    SALOONDOOR_R,

    /**
     * Two radial hands attached at the midpoints of the top and bottom halves
     * sweep from right to left
     */
    WINDSHIELD_R(261),

    /**
     * Two radial hands attached at the midpoints of the left and right halves
     * sweep from top to bottom
     */
    WINDSHIELD_U,

    /**
     * Two sets of radial hands attached at the midpoints of the top and bottom
     * halves sweep from top to bottom and bottom to top
     */
    WINDSHIELD_V,

    /**
     * Two sets of radial hands attached at the midpoints of the left and right
     * halves sweep from left to right and right to left
     */
    WINDSHIELD_H,

    /** Crossfade */
    CROSSFADE(512);

    static {
        int currentValue = 0;
        for (GESVideoStandardTransitionType type: GESVideoStandardTransitionType.values()) {
            if (-1==type.intValue()) {
                type.value = currentValue++;
            } else {
                currentValue = type.value+1;
            }
        }
    }

    private int value;
    private GESVideoStandardTransitionType() {
        value = -1;
    }

    private GESVideoStandardTransitionType(int value) {
        this.value = value;
    }

    @Override
    public int intValue() {
        return value;
    }
    
    public static GESVideoStandardTransitionType fromValue(int value) {
        //TODO: binary search
        for (GESVideoStandardTransitionType type:GESVideoStandardTransitionType.values()) {
            if (type.intValue() == value) {
                return type;
            }
        }
        return NONE;
    }
}
