package org.gstreamer.ges.clip;

import org.gstreamer.lowlevel.GESTextOverlayClipAPI;

import javafx.scene.paint.Color;

/**
 * GESTextOverlayClip
 *
 * GESTextOverlayClip — Render text onto another stream in a GESLayer
 *
 * Description
 *
 * Renders text onto the next lower priority stream using textrender.
 *
 */

/**
 * Properties
 *
 * guint color Read / Write / Construct
 *
 * gchar * font-desc Read / Write /Construct
 *
 * GESTextHAlign halignment Read / Write / Construct
 *
 * gchar * text Read / Write / Construct
 *
 * GESTextVAlign valignment Read / Write / Construct
 *
 * gdouble xpos Read / Write / Construct
 *
 * gdouble ypos Read / Write / Construct
 *
 * GESTextOverlayClip Object Hierarchy
 */
/**
 * Property Details
 *
 * The “color” property
 *
 * “color” guint The color of the text
 *
 * Flags: Read / Write / Construct
 *
 * Default value: 4294967295
 *
 * The “font-desc” property
 *
 * “font-desc” gchar * Pango font description string
 *
 * Flags: Read / Write / Construct
 *
 * Default value: "Serif 36"
 *
 * The “halignment” property
 *
 * “halignment” GESTextHAlign Horizontal alignment of the text
 *
 * Flags: Read / Write / Construct
 *
 * Default value: GES_TEXT_HALIGN_CENTER
 *
 * The “text” property
 *
 * “text” gchar * The text to diplay
 *
 * Flags: Read / Write / Construct
 *
 * Default value: ""
 *
 * The “valignment” property
 *
 * “valignment” GESTextVAlign Vertical alignent of the text
 *
 * Flags: Read / Write / Construct
 *
 * Default value: GES_TEXT_VALIGN_BASELINE
 *
 * The “xpos” property
 *
 * “xpos” gdouble The horizontal position of the text
 *
 * Flags: Read / Write / Construct
 *
 * Allowed values: [0,1]
 *
 * Default value: 0.5
 *
 * The “ypos” property
 *
 * “ypos” gdouble The vertical position of the text
 *
 * Flags: Read / Write / Construct
 *
 * Allowed values: [0,1]
 *
 * Default value: 0.5
 */
public class GESTextOverlayClip extends GESOverlayClip {
    public static final String GTYPE_NAME = "GESTextOverlayClip";
    private static final GESTextOverlayClipAPI ges = GESTextOverlayClipAPI.GESTEXTOVERLAYCLIP_API;

    public GESTextOverlayClip(Initializer init) {
        super(init);
    }

    /**
     * Creates a new GESTextOverlayClip
     *
     * Returns
     *
     * The newly created GESTextOverlayClip, or NULL if there was an error.
     */
    public GESTextOverlayClip() {
        this(initializer(ges.ptr_ges_text_overlay_clip_new(), false, true));
    }

    /**
     * Sets the text this clip will render.
     *
     * Parameters
     *
     * self the GESTextOverlayClip* to set text on
     *
     * text the text to render. an internal copy of this text will be made.
     */
    public void setText(String text) {
        ges.ges_text_overlay_clip_set_text(this, text);
    }

    /**
     * Sets the pango font description of the text
     *
     * Parameters
     *
     * self the GESTextOverlayClip*
     *
     * font_desc the pango font description
     */
    public void setFontDescription(String font_desc) {
        ges.ges_text_overlay_clip_set_font_desc(this, font_desc);
    }
    /**
     * Sets the vertical aligment of the text.
     *
     * Parameters
     *
     * self the GESTextOverlayClip* to set vertical alignement of text on
     *
     * valign GESTextVAlign
     */

    /** Vertical alignment of the text. */
    public enum GESTextVAlign {
        /** draw text on the baseline */
        BASELINE,

        /** draw text on the bottom */
        BOTTOM,

        /** draw text on top */
        TOP,

        /** draw text on ypos position */
        POSITION,

        /** draw text on the center */
        CENTER,
    };

    public void setVAlign(GESTextVAlign valign) {
        ges.ges_text_overlay_clip_set_valign(this, valign);
    }

    /**
     * Sets the horizontal aligment of the text.
     *
     * Parameters
     *
     * self the GESTextOverlayClip* to set horizontal alignement of text on
     *
     * halign GESTextHAlign
     */

    /** Horizontal alignment of the text. */
    public enum GESTextHAlign {
        /** align text left */
        LEFT,
        /** align text left */
        CENTER,
        /** align text right */
        RIGHT,
        /** align text on xpos position */
        POSITION
    };

    public void setHAlign(GESTextHAlign halign) {
        ges.ges_text_overlay_clip_set_halign(this, halign);
    }

    /**
     * Sets the color of the text.
     *
     * Parameters
     *
     * self the GESTextOverlayClip* to set
     *
     * color The color self is being set to
     */
    public void setColor(Color color) {
        int alpha = (int) Math.round(color.getOpacity() * 255);
        int red = (int) Math.round(color.getRed() * 255);
        int blue = (int) Math.round(color.getBlue() * 255);
        int green = (int) Math.round(color.getGreen() * 255);
        int colorValue = (alpha << 24) | (red << 16) | (green << 8) | (blue);
        ges.ges_text_overlay_clip_set_color(this, colorValue);
    }

    /**
     *
     * Sets the horizontal position of the text.
     *
     * Parameters
     *
     * self the GESTextOverlayClip* to set
     *
     * position The horizontal position self is being set to
     */
    public void setX(double position) {
        ges.ges_text_overlay_clip_set_xpos(this, position);
    }

    /**
     * Sets the vertical position of the text.
     *
     * Parameters
     *
     * self the GESTextOverlayClip* to set
     *
     * position The vertical position self is being set to
     */

    public void setY(double position) {
        ges.ges_text_overlay_clip_set_ypos(this, position);
    }

    /**
     * Get the text currently set on self .
     *
     * Parameters
     *
     * self a GESTextOverlayClip
     *
     * Returns
     *
     * The text currently set on self .
     */
    public String getText() {
        return ges.ges_text_overlay_clip_get_text(this);
    }

    /**
     * Get the pango font description used by self .
     *
     * Parameters
     *
     * self a GESTextOverlayClip
     *
     * Returns
     *
     * The pango font description used by self .
     */
    public String getFontDescription() {
        return ges.ges_text_overlay_clip_get_font_desc(this);
    }

    /**
     * Get the vertical aligment used by self .
     *
     * Parameters
     *
     * self a GESTextOverlayClip
     *
     * Returns
     *
     * The vertical aligment used by self .
     */
    public GESTextVAlign getVAlign() {
        return ges.ges_text_overlay_clip_get_valignment(this);
    }

    /**
     * Get the horizontal aligment used by self .
     *
     * Parameters
     *
     * self a GESTextOverlayClip
     *
     * Returns
     *
     * The horizontal aligment used by self .
     */
    public GESTextHAlign getHAlign() {
        return ges.ges_text_overlay_clip_get_halignment(this);
    }

    /**
     * Get the color used by source .
     *
     * Parameters
     *
     * self a GESTextOverlayClip
     *
     * Returns
     *
     * The color used by source .
     */
    public Color getColor() {
        int colorValue = ges.ges_text_overlay_clip_get_color(this);
        int alpha = (colorValue >> 24) & 0xFF;
        int red = (colorValue >> 16) & 0xFF;
        int green = (colorValue >> 8) & 0xFF;
        int blue = (colorValue >> 0) & 0xFF;
        return new Color(red / 255.0, green / 255.0, blue / 255.0, alpha / 255.0);
    }

    /**
     * Get the horizontal position used by source .
     *
     * Parameters
     *
     * self a GESTextOverlayClip
     *
     * Returns
     *
     * The horizontal position used by source .
     */
    public double getX() {
        return ges.ges_text_overlay_clip_get_xpos(this);
    }

    /**
     * Get the vertical position used by source .
     *
     * Parameters
     *
     * self a GESTextOverlayClip
     *
     * Returns The vertical position used by source .
     */
    public double getY() {
        return ges.ges_text_overlay_clip_get_ypos(this);
    }
}
