package org.gstreamer.ges.clip;

import org.gstreamer.ges.GESVideoStandardTransitionType;
import org.gstreamer.lowlevel.GESTransitionClipAPI;

/**
 * GESTransitionClip
 *
 * GESTransitionClip— Transition from one clip to another in a GESLayer
 */

/**
 * Creates an object that mixes together the two underlying objects, A and B.
 * The A object is assumed to have a
 *
 * higher prioirity (lower number) than the B object. At the transition in
 * point, only A will be visible, and by the end only B will be visible.
 *
 * The shape of the video transition depends on the value of the "vtype"
 * property. The default value is "crossfade". For audio, only "crossfade" is
 * supported.
 *
 * The ID of the ExtractableType is the nickname of the vtype property value.
 * Note that this value can be changed after creation and the
 * GESExtractable.asset value will be updated when needed.
 *
 *
 */
/**
 * The “vtype” property
 *
 * “vtype” GESVideoStandardTransitionType a GESVideoStandardTransitionType
 * representing the wipe to use
 *
 * Flags: Read / Write / Construct
 *
 * Default value: GES_VIDEO_STANDARD_TRANSITION_TYPE_CROSSFADE
 */
public class GESTransitionClip extends GESBaseTransitionClip {
    public static final String GTYPE_NAME = "GESTransitionClip";
    private static final GESTransitionClipAPI ges = GESTransitionClipAPI.GESVIDEOTRANSITIONCLIP_API;

    public GESTransitionClip(Initializer init) {
        super(init);
    }

    /**
     * Creates a new GESTransitionClip.
     *
     * Parameters
     *
     * vtype the type of transition to create Returns
     *
     * a newly created GESTransitionClip, or NULL if something went wrong.
     *
     */
    public GESTransitionClip(GESVideoStandardTransitionType vtype) {
        this(initializer(ges.ptr_ges_transition_clip_new(vtype), false, true));
    }

    /**
     * GESTransitionClip for the provided nick .
     *
     * Parameters
     *
     * nick a string representing the type of transition to create Returns
     *
     * The newly created GESTransitionClip, or NULL if something went wrong
     */

    public GESTransitionClip(String nick) {
        this(initializer(ges.ptr_ges_transition_clip_new_for_nick(nick), false, true));
    }
}
