package org.gstreamer.ges.operation;

import org.gstreamer.lowlevel.GESEffectAPI;

/**
 * GESEffect
 *
 * GESEffect — adds an effect build from a parse-launch style bin description to
 * a stream in a GESSourceClip or a GESLayer
 *
 */
/*
 * Object Hierarchy
 *
 * GObject ╰── GInitiallyUnowned ╰── GESTimelineElement ╰── GESTrackElement ╰──
 * GESOperation ╰── GESBaseEffect ╰── GESEffect
 */

public class GESEffect extends GESBaseEffect {
    public static final String GTYPE_NAME = "GESEffect";

    private static final GESEffectAPI ges = GESEffectAPI.GSTEFFECT_API;

    public GESEffect(Initializer init) {
        super(init);
    }

    public GESEffect(String description) {
        this(initializer(ges.ptr_ges_effect_new(description), false, true));
    }

    /*
     *
     * GESEffect * ges_effect_new (const gchar *bin_description);
     *
     * Creates a new GESEffect from the description of the bin. It should be
     * possible to determine the type of the effect through the element 'klass'
     * metadata of the GstElements that will be created. In that corner case,
     * you should use: ges_asset_request26 (GES_TYPE_EFFECT,
     * "audio your ! bin ! description", NULL); and extract that asset to be in
     * full control. Parameters
     *
     * bin_description
     *
     *
     * The gst-launch like bin description of the effect
     *
     * Returns
     *
     * a newly created GESEffect, or NULL if something went wrong. Types and
     * Values GESEffect
     *
     * typedef struct _GESEffect GESEffect;
     *
     * Property Details The “bin-description” property
     *
     * “bin-description” gchar *
     *
     * The description of the effect bin with a gst-launch-style pipeline
     * description.
     *
     * Example: "videobalance saturation=1.5 hue=+0.5"
     *
     * Flags: Read / Write / Construct Only
     *
     * Default value: NULL
     */
}
