package org.gstreamer.ges.operation;

import org.gstreamer.ges.GESVideoStandardTransitionType;
import org.gstreamer.lowlevel.GESVideoTransitionAPI;
import org.gstreamer.lowlevel.GstNative;

/**
 * GESVideoTransition
 *
 * GESVideoTransition—implements video crossfade transition
 *
 */

public class GESVideoTransition extends GESTransition {
    public static final String GTYPE_NAME = "GESVideoTransition";

    private static final GESVideoTransitionAPI ges = GstNative.load("ges", GESVideoTransitionAPI.class);

    public GESVideoTransition(Initializer init) {
        super(init);
    }

    /**
     * Description
     *
     *
     * Creates a new GESVideoTransition.
     *
     * Returns
     *
     * The newly created GESVideoTransition, or NULL if there was an error.
     */
    public GESVideoTransition() {
        this(initializer(ges.ptr_ges_video_transition_new(), false, true));
    }

    public GESVideoTransition(GESVideoStandardTransitionType type) {
        this(initializer(ges.ptr_ges_video_transition_new(), false, true));
        setTransitionType(type);
    }

    /**
     *
     * Sets the transition being used to type . Parameters
     *
     * self
     *
     *
     * a GESVideoTransition
     *
     *
     * type
     *
     *
     * a GESVideoStandardTransitionType
     *
     * Returns
     *
     * TRUE if the transition type was properly changed, else FALSE.
     */
    public boolean setTransitionType(GESVideoStandardTransitionType type) {
        return ges.ges_video_transition_set_transition_type(this, type);
    }
    /**
     * Get the transition type used by trans. Parameters
     *
     * trans
     *
     *
     * a GESVideoTransition49
     *
     * Returns
     *
     * The transition type used by trans .
     */
    public GESVideoStandardTransitionType getTransitionType() {
        return ges.ges_video_transition_get_transition_type(this);
    }

    /**
     * Set the border property of self , this value represents the border width
     * of the transition. In case this value does not make sense for the current
     * transition type, it is cached for later use. Parameters
     *
     * self
     *
     *
     * The GESVideoTransition to set the border to
     *
     *
     * value
     *
     *
     * The value of the border to set on object
     */
    public void setBorder(int value) {
        ges.ges_video_transition_set_border(this, value);
    }

    /**
     * Get the border property of self , this value represents the border width
     * of the transition. Parameters
     *
     * self
     *
     *
     * The GESVideoTransition55 to get the border from
     *
     * Returns
     *
     * The border values of self or -1 if
     *
     * not meaningful (this will happen when not using a smpte transition).
     */
    public int getBorder() {
        return ges.ges_video_transition_get_border(this);
    }

    /**
     * Set the invert property of self , this value represents the direction of
     * the transition. In case this value does not make sense for the current
     * transition type, it is cached for later use. Parameters
     *
     * self
     *
     *
     * The GESVideoTransition to set invert on
     *
     *
     * inverted
     *
     *
     * TRUE if the transition should be inverted FALSE otherwise
     */
    public void setInverted(boolean inverted) {
        ges.ges_video_transition_set_inverted(this, inverted);
    }

     /**
     * Get the invert property of self , this value represents the direction of
     * the transition. Parameters
     *
     * self
     *
     *
     * The GESVideoTransition63 to get the inversion from
     *
     * Returns
     *
     * The invert value of self
     */
    public boolean isInverted() {
        return ges.ges_video_transition_is_inverted(this);
    }
    /*
    * Property Details The “border” property
    *
    * “border” guint64
    *
    * This value represents the border width of the transition.
    *
    * Flags: Read / Write
    *
    * Default value: 0 The “invert” property
    *
    * “invert” gboolean65
    *
    * This value represents the direction of the transition.
    *
    * Flags: Read / Write
    *
    * Default value: FALSE The “transition-type” property
    *
    * “transition-type” GESVideoStandardTransitionType66
    *
    * The type of the transition.
    *
    * Flags: Read / Write
    *
    * Default value: GES_VIDEO_STANDARD_TRANSITION_TYPE_CROSSFADE Generated by
    * GTK-Doc V1.24
    */
}
