package org.gstreamer.javafx;

import java.nio.ByteBuffer;

import org.gstreamer.Buffer;
import org.gstreamer.Caps;
import org.gstreamer.Structure;

import com.sun.media.jfxmedia.control.VideoDataBuffer;
import com.sun.media.jfxmedia.control.VideoFormat;

public class GstVideoDataBuffer implements VideoDataBuffer {
    //FIXME: change to private
    public Buffer buffer;
    private VideoFormat videoFormat;
    private int width;
    private int height;
    private ByteBuffer bb[];
    private long time;
    private int planesCount;
    private int stride;
    
    public GstVideoDataBuffer(Buffer buffer, Caps caps, long time) {
        this.buffer = buffer;
        this.time = time;
        buffer.ref();

        Structure struct = caps.getStructure(0);
        width = struct.getInteger("width");
        height = struct.getInteger("height");

        String gstFormat = struct.getString("format");

        //TODO: lazy buffer obtaining as not every VBD have to be rendered. 
        if (gstFormat.equals("I420")) {
            videoFormat = VideoFormat.YCbCr_420p;
            planesCount = 3;
            stride = width;
            bb = new ByteBuffer[planesCount];
            bb[0] = buffer.getByteBuffer(0);
            int offset = width * height;
            bb[2] = buffer.getByteBuffer(offset);
            offset += (width / 2) * (height / 2);
            bb[1] = buffer.getByteBuffer(offset);
        } else if (gstFormat.equals("BGRA") || true/* do best for rest of formats as BGRA */) {
            videoFormat = VideoFormat.BGRA_PRE;
            planesCount = 1;
            stride = width;
            bb = new ByteBuffer[planesCount];
            bb[0] = buffer.getByteBuffer(0);
        }
        refs = 1;
    }

    @Override
    public ByteBuffer getBufferForPlane(int plane) {
        ByteBuffer res = null;
        if (VideoDataBuffer.PACKED_FORMAT_PLANE == plane) { /* PACKED_FORMAT_PLANE == YCBCR_PLANE_LUMA */
            res = bb[0];
        } else if (VideoDataBuffer.YCBCR_PLANE_CR == plane) {
            res = bb[1];
        } else if (VideoDataBuffer.YCBCR_PLANE_CB == plane) {
            res = bb[2];
        }
        return res;
    }

    @Override
    public double getTimestamp() {
        return time;
    }

    @Override
    public int getWidth() {
        return width;
    }

    @Override
    public int getHeight() {
        return height;
    }

    @Override
    public int getEncodedWidth() {
        return width;
    }

    @Override
    public int getEncodedHeight() {
        return height;
    }

    @Override
    public VideoFormat getFormat() {
        return videoFormat;
    }

    @Override
    public boolean hasAlpha() {
        //TODO: check if it is used somewhere (by prism). 
        return false;
    }

    @Override
    public int getPlaneCount() {
        return planesCount;
    }

    @Override
    public int getStrideForPlane(int planeIndex) {
        int stride = 0;
        if (VideoDataBuffer.PACKED_FORMAT_PLANE == planeIndex) { /* PACKED_FORMAT_PLANE == YCBCR_PLANE_LUMA */
            stride = this.stride;
        } else if (VideoDataBuffer.YCBCR_PLANE_CR == planeIndex) {
            stride = this.stride / 2;
        } else if (VideoDataBuffer.YCBCR_PLANE_CB == planeIndex) {
            stride = this.stride / 2;
        }
        return stride;
    }

    @Override
    public int[] getPlaneStrides() {
        int[] strides = new int[getPlaneCount()];
        for (int i = 0; i < strides.length; i++) {
            strides[i] = getStrideForPlane(i);
        }
        return strides;
    }

    @Override
    public VideoDataBuffer convertToFormat(VideoFormat newFormat) {
        return null;
    }

    @Override
    public void setDirty() {

    }

    private volatile int refs;

    @Override
    public synchronized void holdFrame() {
        assert(0 < refs);

        refs++;
    }

    @Override
    public synchronized void releaseFrame() {
        if (--refs <= 0) {
            if (null != buffer) {
                buffer.unref();
                buffer = null;
            }
        }
    }
}