package org.gstreamer.javafx;

import java.nio.ByteBuffer;
import java.util.concurrent.TimeUnit;

import org.gstreamer.Buffer;
import org.gstreamer.Caps;
import org.gstreamer.Element;
import org.gstreamer.ElementFactory;
import org.gstreamer.Pipeline;
import org.gstreamer.Sample;
import org.gstreamer.State;
import org.gstreamer.Structure;
import org.gstreamer.elements.AppSink;
import org.gstreamer.elements.AppSrc;
import org.gstreamer.elements.AppSrc.NEED_DATA;

import javafx.scene.image.Image;
import javafx.scene.image.PixelFormat;
import javafx.scene.image.WritableImage;

public class ImageConverter {
    public ImageConverter() {

    }

    private void log(String msg) {
        System.out.println("[IC]" + msg);
    }

    private Image convertPipeline(Buffer buffer, Caps caps) {
        Pipeline pipeline = new Pipeline("converter_pipeline");
        AppSrc appSrc = (AppSrc) ElementFactory.make("appsrc", "converter_appsrc");
        appSrc.setCaps(caps);
        AppSink appSink = (AppSink) ElementFactory.make("appsink", "converter_appsink");
        Caps sinkCaps = Caps.fromString("video/x-raw, format=BGRA");
        appSink.setCaps(sinkCaps);
        Element converter = ElementFactory.make("videoconvert", "converter_videoconvert");

        NEED_DATA listener = (elem, size) -> {
            log(" need data");
            elem.pushBuffer(buffer);
        };
        appSrc.connect(listener);

        pipeline.addMany(appSrc, converter, appSink);
        Element.linkMany(appSrc, converter, appSink);

        pipeline.pause();

        Image image = createImageFromSampleOfAppSink(appSink, null);
        log(" image -> " + image);
        pipeline.setState(State.NULL);
        return image;
    }

    public static WritableImage createImageFromSampleOfAppSink(AppSink appSink, WritableImage oldImage) {
        WritableImage image = null;
        Sample preroll = appSink.pullPreroll();

        try {
            if (null != preroll) {
                Buffer buffer = preroll.getBuffer();
                Caps caps = preroll.getCaps();
                if (null != caps && null != buffer) {

                    Structure struct = caps.getStructure(0);
                    int w = struct.getInteger("width");
                    int h = struct.getInteger("height");

                    ByteBuffer bb = buffer.getByteBuffer();

                    if (null != bb) {
                        if (null != oldImage && 0 == Double.compare(oldImage.getWidth(), w) && 0 == Double.compare(oldImage.getHeight(), h)) {
                            image = oldImage;
                        } else {
                            image = new WritableImage(w, h);
                        }
                        image.getPixelWriter().setPixels(0, 0, w, h, PixelFormat.getByteBgraInstance(), bb, w * 4);
                    }
                    return image;
                }
            }
        } finally {
            preroll.dispose();
        }

        return image;
    }

    public Image convertBufferToImage(Buffer buffer, int width, int height) {
        //long time = System.nanoTime();
        Buffer buf = buffer.copy();
        Caps caps = Caps.fromString("video/x-raw,format=I420,width=" + width + ",height=" + height);
        Image img = convertPipeline(buf, caps);
        //time = System.nanoTime() - time;
        //System.out.println(" converting buffer took:"+TimeUnit.NANOSECONDS.toMillis(time) + " ms");
        return img;
    }
}
